@@ -144,7 +144,7 @@ virDomainSnapshotDiskDefParseXML(xmlNodePtr node,
     char *type = NULL;
     char *driver = NULL;
     xmlNodePtr cur;
-    xmlNodePtr saved = ctxt->node;
+    VIR_XPATH_NODE_AUTORESTORE(ctxt);
 
     ctxt->node = node;
 
@@ -205,7 +205,6 @@ virDomainSnapshotDiskDefParseXML(xmlNodePtr node,
 
     ret = 0;
  cleanup:
-    ctxt->node = saved;
 
     VIR_FREE(driver);
     VIR_FREE(snapshot);
@@ -232,7 +231,7 @@ virDomainSnapshotDefParse(xmlXPathContextPtr ctxt,
     xmlNodePtr inactiveDomNode = NULL;
     size_t i;
     int n;
-    char *creation = NULL, *state = NULL;
+    char *state = NULL;
     int active;
     char *tmp;
     char *memorySnapshot = NULL;
@@ -405,7 +404,6 @@ virDomainSnapshotDefParse(xmlXPathContextPtr ctxt,
     ret = g_steal_pointer(&def);
 
  cleanup:
-    VIR_FREE(creation);
     VIR_FREE(state);
     VIR_FREE(nodes);
     VIR_FREE(memorySnapshot);
