@@ -23,8 +23,6 @@
 
 #include <config.h>
 
-#include <netdb.h>
-
 #include "internal.h"
 #include "datatypes.h"
 #include "viralloc.h"
@@ -34,6 +32,7 @@
 #include "esx_private.h"
 #include "esx_util.h"
 #include "virstring.h"
+#include "virsocket.h"
 
 #define VIR_FROM_THIS VIR_FROM_ESX
 
