@@ -535,8 +535,7 @@ virDomainCheckpointDelete(virDomainCheckpointPtr checkpoint,
 int
 virDomainCheckpointRef(virDomainCheckpointPtr checkpoint)
 {
-    VIR_DEBUG("checkpoint=%p, refs=%d", checkpoint,
-              checkpoint ? checkpoint->parent.u.s.refs : 0);
+    VIR_DEBUG("checkpoint=%p", checkpoint);
 
     virResetLastError();
 
