@@ -38,7 +38,13 @@
 #include "libxl_capabilities.h"
 #include "libxl_logger.h"
 
-#define LIBXL_DRIVER_NAME "xenlight"
+#define LIBXL_DRIVER_EXTERNAL_NAME "Xen"
+/*
+ * We are stuck with the 'xenlight' name since it is used by the hostdev
+ * manager. Changing it would break management of any host devices previously
+ * managed under the name 'xenlight'.
+ */
+#define LIBXL_DRIVER_INTERNAL_NAME "xenlight"
 #define LIBXL_VNC_PORT_MIN  5900
 #define LIBXL_VNC_PORT_MAX  65535
 
