@@ -41,7 +41,9 @@ struct _qemuAgentCallbacks {
 
 qemuAgentPtr qemuAgentOpen(virDomainObjPtr vm,
                            const virDomainChrSourceDef *config,
-                           qemuAgentCallbacksPtr cb);
+                           GMainContext *context,
+                           qemuAgentCallbacksPtr cb,
+                           bool singleSync);
 
 void qemuAgentClose(qemuAgentPtr mon);
 
@@ -96,7 +98,9 @@ int qemuAgentShutdown(qemuAgentPtr mon,
 int qemuAgentFSFreeze(qemuAgentPtr mon,
                       const char **mountpoints, unsigned int nmountpoints);
 int qemuAgentFSThaw(qemuAgentPtr mon);
-int qemuAgentGetFSInfo(qemuAgentPtr mon, qemuAgentFSInfoPtr **info);
+int qemuAgentGetFSInfo(qemuAgentPtr mon,
+                       qemuAgentFSInfoPtr **info,
+                       bool report_unsupported);
 
 int qemuAgentSuspend(qemuAgentPtr mon,
                      unsigned int target);
@@ -127,7 +131,8 @@ int qemuAgentUpdateCPUInfo(unsigned int nvcpus,
 
 int
 qemuAgentGetHostname(qemuAgentPtr mon,
-                     char **hostname);
+                     char **hostname,
+                     bool report_unsupported);
 
 int qemuAgentGetTime(qemuAgentPtr mon,
                      long long *seconds,
@@ -148,17 +153,20 @@ int qemuAgentSetUserPassword(qemuAgentPtr mon,
 int qemuAgentGetUsers(qemuAgentPtr mon,
                       virTypedParameterPtr *params,
                       int *nparams,
-                      int *maxparams);
+                      int *maxparams,
+                      bool report_unsupported);
 
 int qemuAgentGetOSInfo(qemuAgentPtr mon,
                        virTypedParameterPtr *params,
                        int *nparams,
-                       int *maxparams);
+                       int *maxparams,
+                       bool report_unsupported);
 
 int qemuAgentGetTimezone(qemuAgentPtr mon,
                          virTypedParameterPtr *params,
                          int *nparams,
-                         int *maxparams);
+                         int *maxparams,
+                         bool report_unsupported);
 
 void qemuAgentSetResponseTimeout(qemuAgentPtr mon,
                                  int timeout);
