@@ -242,6 +242,14 @@ qemuSlirpStop(qemuSlirpPtr slirp,
 }
 
 
+int
+qemuSlirpSetupCgroup(qemuSlirpPtr slirp,
+                     virCgroupPtr cgroup)
+{
+    return virCgroupAddProcess(cgroup, slirp->pid);
+}
+
+
 int
 qemuSlirpStart(qemuSlirpPtr slirp,
                virDomainObjPtr vm,
@@ -249,6 +257,7 @@ qemuSlirpStart(qemuSlirpPtr slirp,
                virDomainNetDefPtr net,
                bool incoming)
 {
+    qemuDomainObjPrivatePtr priv = vm->privateData;
     g_autoptr(virQEMUDriverConfig) cfg = virQEMUDriverGetConfig(driver);
     g_autoptr(virCommand) cmd = NULL;
     g_autofree char *pidfile = NULL;
@@ -258,6 +267,7 @@ qemuSlirpStart(qemuSlirpPtr slirp,
     int exitstatus = 0;
     int cmdret = 0;
     VIR_AUTOCLOSE errfd = -1;
+    bool killDBusDaemon = false;
 
     if (incoming &&
         !qemuSlirpHasFeature(slirp, QEMU_SLIRP_FEATURE_MIGRATE)) {
@@ -306,6 +316,9 @@ qemuSlirpStart(qemuSlirpPtr slirp,
         g_autofree char *id = qemuSlirpGetDBusVMStateId(net);
         g_autofree char *dbus_addr = qemuDBusGetAddress(driver, vm);
 
+        /* If per VM DBus daemon is not running yet, start it
+         * now. But if we fail later on, make sure to kill it. */
+        killDBusDaemon = !QEMU_DOMAIN_PRIVATE(vm)->dbusDaemonRunning;
         if (qemuDBusStart(driver, vm) < 0)
             return -1;
 
@@ -348,6 +361,10 @@ qemuSlirpStart(qemuSlirpPtr slirp,
     }
 
     slirp->pid = pid;
+
+    if (priv->cgroup && qemuSlirpSetupCgroup(slirp, priv->cgroup) < 0)
+        goto error;
+
     return 0;
 
  error:
@@ -355,6 +372,8 @@ qemuSlirpStart(qemuSlirpPtr slirp,
         virProcessKillPainfully(pid, true);
     if (pidfile)
         unlink(pidfile);
-    qemuDBusStop(driver, vm);
+    if (killDBusDaemon)
+        qemuDBusStop(driver, vm);
+    slirp->pid = 0;
     return -1;
 }
