@@ -21,7 +21,6 @@
 
 #include <config.h>
 
-#include <sys/wait.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -34,6 +33,7 @@
 #include "virlog.h"
 #include "virfile.h"
 #include "virstring.h"
+#include "virutil.h"
 #include "storage_util.h"
 
 #define VIR_FROM_THIS VIR_FROM_STORAGE
