@@ -29,7 +29,6 @@
 #include "virlog.h"
 #include "virstring.h"
 #include "virfile.h"
-#include "virutil.h"
 
 #define VIR_FROM_THIS VIR_FROM_NONE
 
@@ -103,16 +102,16 @@ static int testFDStreamReadCommon(const char *scratchdir, bool blocking)
                     g_usleep(20 * 1000);
                     goto reread;
                 }
-                virFilePrintf(stderr, "Failed to read stream: %s\n",
-                              virGetLastErrorMessage());
+                fprintf(stderr, "Failed to read stream: %s\n",
+                        virGetLastErrorMessage());
                 goto cleanup;
             }
             if (got == 0) {
                 /* Expect EOF 1/2 through last pattern */
                 if (i == 9 && want == (PATTERN_LEN / 2))
                     break;
-                virFilePrintf(stderr, "Unexpected EOF block %zu want %zu\n",
-                              i, want);
+                fprintf(stderr, "Unexpected EOF block %zu want %zu\n",
+                        i, want);
                 goto cleanup;
             }
             offset += got;
@@ -120,25 +119,25 @@ static int testFDStreamReadCommon(const char *scratchdir, bool blocking)
         }
         if (i == 0) {
             if (memcmp(buf, pattern + (PATTERN_LEN / 2), PATTERN_LEN / 2) != 0) {
-                virFilePrintf(stderr, "Mismatched pattern data iteration %zu\n", i);
+                fprintf(stderr, "Mismatched pattern data iteration %zu\n", i);
                 goto cleanup;
             }
         } else if (i == 9) {
             if (memcmp(buf, pattern, PATTERN_LEN / 2) != 0) {
-                virFilePrintf(stderr, "Mismatched pattern data iteration %zu\n", i);
+                fprintf(stderr, "Mismatched pattern data iteration %zu\n", i);
                 goto cleanup;
             }
         } else {
             if (memcmp(buf, pattern, PATTERN_LEN) != 0) {
-                virFilePrintf(stderr, "Mismatched pattern data iteration %zu\n", i);
+                fprintf(stderr, "Mismatched pattern data iteration %zu\n", i);
                 goto cleanup;
             }
         }
     }
 
     if (st->driver->streamFinish(st) != 0) {
-        virFilePrintf(stderr, "Failed to finish stream: %s\n",
-                      virGetLastErrorMessage());
+        fprintf(stderr, "Failed to finish stream: %s\n",
+                virGetLastErrorMessage());
         goto cleanup;
     }
 
@@ -226,8 +225,8 @@ static int testFDStreamWriteCommon(const char *scratchdir, bool blocking)
                 if (i == 9 &&
                     want == (PATTERN_LEN / 2))
                     break;
-                virFilePrintf(stderr, "Failed to write stream: %s\n",
-                              virGetLastErrorMessage());
+                fprintf(stderr, "Failed to write stream: %s\n",
+                        virGetLastErrorMessage());
                 goto cleanup;
             }
             offset += got;
@@ -236,8 +235,8 @@ static int testFDStreamWriteCommon(const char *scratchdir, bool blocking)
     }
 
     if (st->driver->streamFinish(st) != 0) {
-        virFilePrintf(stderr, "Failed to finish stream: %s\n",
-                      virGetLastErrorMessage());
+        fprintf(stderr, "Failed to finish stream: %s\n",
+                virGetLastErrorMessage());
         goto cleanup;
    }
 
@@ -252,9 +251,9 @@ static int testFDStreamWriteCommon(const char *scratchdir, bool blocking)
             want = PATTERN_LEN;
 
         if ((got = saferead(fd, buf, want)) != want) {
-            virFilePrintf(stderr,
-                          "Short read from data, i=%zu got=%zu want=%zu\n",
-                          i, got, want);
+            fprintf(stderr,
+                    "Short read from data, i=%zu got=%zu want=%zu\n",
+                    i, got, want);
             goto cleanup;
         }
 
@@ -262,22 +261,22 @@ static int testFDStreamWriteCommon(const char *scratchdir, bool blocking)
             size_t j;
             for (j = 0; j < (PATTERN_LEN / 2); j++) {
                 if (buf[j] != 0) {
-                    virFilePrintf(stderr, "Mismatched pattern data iteration %zu\n", i);
+                    fprintf(stderr, "Mismatched pattern data iteration %zu\n", i);
                     goto cleanup;
                 }
             }
             if (memcmp(buf + (PATTERN_LEN / 2), pattern, PATTERN_LEN / 2) != 0) {
-                virFilePrintf(stderr, "Mismatched pattern data iteration %zu\n", i);
+                fprintf(stderr, "Mismatched pattern data iteration %zu\n", i);
                 goto cleanup;
             }
         } else if (i == 9) {
             if (memcmp(buf, pattern, PATTERN_LEN / 2) != 0) {
-                virFilePrintf(stderr, "Mismatched pattern data iteration %zu\n", i);
+                fprintf(stderr, "Mismatched pattern data iteration %zu\n", i);
                 goto cleanup;
             }
         } else {
             if (memcmp(buf, pattern, PATTERN_LEN) != 0) {
-                virFilePrintf(stderr, "Mismatched pattern data iteration %zu\n", i);
+                fprintf(stderr, "Mismatched pattern data iteration %zu\n", i);
                 goto cleanup;
             }
         }
@@ -320,7 +319,7 @@ mymain(void)
     int ret = 0;
 
     if (!g_mkdtemp(scratchdir)) {
-        virFilePrintf(stderr, "Cannot create fdstreamdir");
+        fprintf(stderr, "Cannot create fdstreamdir");
         abort();
     }
 
