@@ -59,6 +59,9 @@ struct _qemuMonitorTest {
 
     bool allowUnusedCommands;
 
+    bool skipValidationDeprecated;
+    bool skipValidationRemoved;
+
     char *incoming;
     size_t incomingLength;
     size_t incomingCapacity;
@@ -529,17 +532,13 @@ qemuMonitorTestHandlerDataFree(void *opaque)
 }
 
 
-/* Returns -1 on error, 0 if validation was successful/not necessary, 1 if
- * the validation has failed, and the reply was properly constructed */
+/* Returns -1 on error, 0 if validation was successful/not necessary */
 static int
 qemuMonitorTestProcessCommandDefaultValidate(qemuMonitorTestPtr test,
                                              const char *cmdname,
                                              virJSONValuePtr args)
 {
     g_auto(virBuffer) debug = VIR_BUFFER_INITIALIZER;
-    virJSONValuePtr schemaroot;
-    g_autoptr(virJSONValue) emptyargs = NULL;
-    g_autofree char *schemapath = NULL;
 
     if (!test->qapischema)
         return 0;
@@ -555,20 +554,16 @@ qemuMonitorTestProcessCommandDefaultValidate(qemuMonitorTestPtr test,
     if (STREQ(cmdname, "device_add"))
         return 0;
 
-    schemapath = g_strdup_printf("%s/arg-type", cmdname);
-
-    if (virQEMUQAPISchemaPathGet(schemapath, test->qapischema, &schemaroot) < 0 ||
-        !schemaroot) {
-        qemuMonitorTestError("command '%s' not found in QAPI schema", cmdname);
-        return -1;
-    }
-
-    if (!args)
-        args = emptyargs = virJSONValueNewObject();
-
-    if (testQEMUSchemaValidate(args, schemaroot, test->qapischema, &debug) < 0) {
+    if (testQEMUSchemaValidateCommand(cmdname, args, test->qapischema,
+                                      test->skipValidationDeprecated,
+                                      test->skipValidationRemoved,
+                                      &debug) < 0) {
         if (virTestGetDebug() == 2) {
-            g_autofree char *argstr = virJSONValueToString(args, true);
+            g_autofree char *argstr = NULL;
+
+            if (args)
+                argstr = virJSONValueToString(args, true);
+
             fprintf(stderr,
                     "\nfailed to validate arguments of '%s' against QAPI schema\n"
                     "args:\n%s\nvalidator output:\n %s\n",
@@ -595,7 +590,6 @@ qemuMonitorTestProcessCommandDefault(qemuMonitorTestPtr test,
     g_autoptr(virJSONValue) val = NULL;
     virJSONValuePtr cmdargs = NULL;
     const char *cmdname;
-    int rc;
 
     if (!(val = virJSONValueFromString(cmdstr)))
         return -1;
@@ -606,10 +600,8 @@ qemuMonitorTestProcessCommandDefault(qemuMonitorTestPtr test,
     }
 
     cmdargs = virJSONValueObjectGet(val, "arguments");
-    if ((rc = qemuMonitorTestProcessCommandDefaultValidate(test, cmdname, cmdargs)) < 0)
+    if (qemuMonitorTestProcessCommandDefaultValidate(test, cmdname, cmdargs) < 0)
         return -1;
-    if (rc == 1)
-        return 0;
 
     if (data->command_name && STRNEQ(data->command_name, cmdname)) {
         qemuMonitorTestErrorInvalidCommand(data->command_name, cmdname);
@@ -651,7 +643,6 @@ qemuMonitorTestProcessCommandVerbatim(qemuMonitorTestPtr test,
     virJSONValuePtr cmdargs;
     const char *cmdname;
     int ret = -1;
-    int rc;
 
     /* JSON strings will be reformatted to simplify checking */
     if (!(json = virJSONValueFromString(cmdstr)) ||
@@ -663,12 +654,8 @@ qemuMonitorTestProcessCommandVerbatim(qemuMonitorTestPtr test,
     /* in this case we do a best-effort schema check if we can find the command */
     if ((cmdname = virJSONValueObjectGetString(json, "execute"))) {
         cmdargs = virJSONValueObjectGet(json, "arguments");
-
-        if ((rc = qemuMonitorTestProcessCommandDefaultValidate(test, cmdname, cmdargs)) < 0)
+        if (qemuMonitorTestProcessCommandDefaultValidate(test, cmdname, cmdargs) < 0)
             return -1;
-
-        if (rc == 1)
-            return 0;
     }
 
     if (STREQ(data->command_name, cmdstr)) {
@@ -1317,6 +1304,31 @@ qemuMonitorTestAllowUnusedCommands(qemuMonitorTestPtr test)
 }
 
 
+/**
+ * qemuMonitorTestSkipDeprecatedValidation:
+ * @test: test monitor object
+ * @allowRemoved: don't produce errors if command was removed from QMP schema
+ *
+ * By default if the QMP schema is provided all test items/commands are
+ * validated against the schema. This function allows to override the validation
+ * and additionally if @allowRemoved is true and if such a command is no longer
+ * present in the QMP, only a warning is printed.
+ *
+ * '@allowRemoved' must be used only if a suitable replacement is already in
+ * use and the code tests legacy interactions.
+ *
+ * Note that currently '@allowRemoved' influences only removed commands. If an
+ * argument is removed it will still fail validation.
+ */
+void
+qemuMonitorTestSkipDeprecatedValidation(qemuMonitorTestPtr test,
+                                        bool allowRemoved)
+{
+    test->skipValidationDeprecated = true;
+    test->skipValidationRemoved = allowRemoved;
+}
+
+
 static int
 qemuMonitorTestFullAddItem(qemuMonitorTestPtr test,
                            const char *filename,
