@@ -21,16 +21,22 @@
 #include "testutilsqemuschema.h"
 #include "qemu/qemu_qapi.h"
 
+struct testQEMUSchemaValidateCtxt {
+    virHashTablePtr schema;
+    virBufferPtr debug;
+    bool allowDeprecated;
+};
+
+
 static int
 testQEMUSchemaValidateRecurse(virJSONValuePtr obj,
                               virJSONValuePtr root,
-                              virHashTablePtr schema,
-                              virBufferPtr debug);
+                              struct testQEMUSchemaValidateCtxt *ctxt);
 
 static int
 testQEMUSchemaValidateBuiltin(virJSONValuePtr obj,
                               virJSONValuePtr root,
-                              virBufferPtr debug)
+                              struct testQEMUSchemaValidateCtxt *ctxt)
 {
     const char *t = virJSONValueObjectGetString(root, "json-type");
     const char *s = NULL;
@@ -81,17 +87,16 @@ testQEMUSchemaValidateBuiltin(virJSONValuePtr obj,
 
  cleanup:
     if (ret == 0)
-        virBufferAsprintf(debug, "'%s': OK", s);
+        virBufferAsprintf(ctxt->debug, "'%s': OK", s);
     else
-        virBufferAsprintf(debug, "ERROR: expected type '%s', actual type %d",
+        virBufferAsprintf(ctxt->debug, "ERROR: expected type '%s', actual type %d",
                           t, virJSONValueGetType(obj));
     return ret;
 }
 
 struct testQEMUSchemaValidateObjectMemberData {
     virJSONValuePtr rootmembers;
-    virHashTablePtr schema;
-    virBufferPtr debug;
+    struct testQEMUSchemaValidateCtxt *ctxt;
     bool missingMandatory;
 };
 
@@ -123,36 +128,32 @@ testQEMUSchemaValidateObjectMember(const char *key,
                                    void *opaque)
 {
     struct testQEMUSchemaValidateObjectMemberData *data = opaque;
-    virJSONValuePtr keymember = NULL;
+    g_autoptr(virJSONValue) keymember = NULL;
     const char *keytype;
     virJSONValuePtr keyschema = NULL;
-    int ret = -1;
+    int rc;
 
-    virBufferStrcat(data->debug, key, ": ", NULL);
+    virBufferStrcat(data->ctxt->debug, key, ": ", NULL);
 
     /* lookup 'member' entry for key */
     if (!(keymember = testQEMUSchemaStealObjectMemberByName(key, data->rootmembers))) {
-        virBufferAddLit(data->debug, "ERROR: attribute not in schema");
-        goto cleanup;
+        virBufferAddLit(data->ctxt->debug, "ERROR: attribute not in schema\n");
+        return -1;
     }
 
     /* lookup schema entry for keytype */
     if (!(keytype = virJSONValueObjectGetString(keymember, "type")) ||
-        !(keyschema = virHashLookup(data->schema, keytype))) {
-        virBufferAsprintf(data->debug, "ERROR: can't find schema for type '%s'",
+        !(keyschema = virHashLookup(data->ctxt->schema, keytype))) {
+        virBufferAsprintf(data->ctxt->debug, "ERROR: can't find schema for type '%s'\n",
                           NULLSTR(keytype));
-        ret = -2;
-        goto cleanup;
+        return -2;
     }
 
     /* recurse */
-    ret = testQEMUSchemaValidateRecurse(value, keyschema, data->schema,
-                                        data->debug);
+    rc = testQEMUSchemaValidateRecurse(value, keyschema, data->ctxt);
 
- cleanup:
-    virBufferAddLit(data->debug, "\n");
-    virJSONValueFree(keymember);
-    return ret;
+    virBufferAddLit(data->ctxt->debug, "\n");
+    return rc;
 }
 
 
@@ -184,20 +185,18 @@ static int
 testQEMUSchemaValidateObjectMergeVariant(virJSONValuePtr root,
                                          const char *variantfield,
                                          const char *variantname,
-                                         virHashTablePtr schema,
-                                         virBufferPtr debug)
+                                         struct testQEMUSchemaValidateCtxt *ctxt)
 {
     size_t i;
-    virJSONValuePtr variants = NULL;
+    g_autoptr(virJSONValue) variants = NULL;
     virJSONValuePtr variant;
     virJSONValuePtr variantschema;
     virJSONValuePtr variantschemamembers;
     virJSONValuePtr rootmembers;
     const char *varianttype = NULL;
-    int ret = -1;
 
     if (!(variants = virJSONValueObjectStealArray(root, "variants"))) {
-        virBufferAddLit(debug, "ERROR: missing 'variants' in schema\n");
+        virBufferAddLit(ctxt->debug, "ERROR: missing 'variants' in schema\n");
         return -2;
     }
 
@@ -212,19 +211,17 @@ testQEMUSchemaValidateObjectMergeVariant(virJSONValuePtr root,
     }
 
     if (!varianttype) {
-        virBufferAsprintf(debug, "ERROR: variant '%s' for discriminator '%s' not found\n",
+        virBufferAsprintf(ctxt->debug, "ERROR: variant '%s' for discriminator '%s' not found\n",
                           variantname, variantfield);
-        goto cleanup;
-
+        return -1;
     }
 
-    if (!(variantschema = virHashLookup(schema, varianttype)) ||
+    if (!(variantschema = virHashLookup(ctxt->schema, varianttype)) ||
         !(variantschemamembers = virJSONValueObjectGetArray(variantschema, "members"))) {
-        virBufferAsprintf(debug,
+        virBufferAsprintf(ctxt->debug,
                           "ERROR: missing schema or schema members for variant '%s'(%s)\n",
                           variantname, varianttype);
-        ret = -2;
-        goto cleanup;
+        return -2;
     }
 
     rootmembers = virJSONValueObjectGetArray(root, "members");
@@ -232,15 +229,10 @@ testQEMUSchemaValidateObjectMergeVariant(virJSONValuePtr root,
     if (virJSONValueArrayForeachSteal(variantschemamembers,
                                       testQEMUSchemaValidateObjectMergeVariantMember,
                                       rootmembers) < 0) {
-        ret = -2;
-        goto cleanup;
+        return -2;
     }
 
-    ret = 0;
-
- cleanup:
-    virJSONValueFree(variants);
-    return ret;
+    return 0;
 }
 
 
@@ -252,7 +244,7 @@ testQEMUSchemaValidateObjectMandatoryMember(size_t pos G_GNUC_UNUSED,
     struct testQEMUSchemaValidateObjectMemberData *data = opaque;
 
     if (virJSONValueObjectHasKey(item, "default") != 1) {
-        virBufferAsprintf(data->debug, "ERROR: missing mandatory attribute '%s'\n",
+        virBufferAsprintf(data->ctxt->debug, "ERROR: missing mandatory attribute '%s'\n",
                           NULLSTR(virJSONValueObjectGetString(item, "name")));
         data->missingMandatory = true;
     }
@@ -264,42 +256,36 @@ testQEMUSchemaValidateObjectMandatoryMember(size_t pos G_GNUC_UNUSED,
 static int
 testQEMUSchemaValidateObject(virJSONValuePtr obj,
                              virJSONValuePtr root,
-                             virHashTablePtr schema,
-                             virBufferPtr debug)
+                             struct testQEMUSchemaValidateCtxt *ctxt)
 {
-    struct testQEMUSchemaValidateObjectMemberData data = { NULL, schema,
-                                                           debug, false };
-    virJSONValuePtr localroot = NULL;
+    struct testQEMUSchemaValidateObjectMemberData data = { NULL, ctxt, false };
+    g_autoptr(virJSONValue) localroot = NULL;
     const char *variantfield;
     const char *variantname;
-    int ret = -1;
 
     if (virJSONValueGetType(obj) != VIR_JSON_TYPE_OBJECT) {
-        virBufferAddLit(debug, "ERROR: not an object");
+        virBufferAddLit(ctxt->debug, "ERROR: not an object");
         return -1;
     }
 
-    virBufferAddLit(debug, "{\n");
-    virBufferAdjustIndent(debug, 3);
+    virBufferAddLit(ctxt->debug, "{\n");
+    virBufferAdjustIndent(ctxt->debug, 3);
 
     /* copy schema */
-    if (!(localroot = virJSONValueCopy(root))) {
-        ret = -2;
-        goto cleanup;
-    }
+    if (!(localroot = virJSONValueCopy(root)))
+        return -2;
 
     /* remove variant */
     if ((variantfield = virJSONValueObjectGetString(localroot, "tag"))) {
         if (!(variantname = virJSONValueObjectGetString(obj, variantfield))) {
-            virBufferAsprintf(debug, "ERROR: missing variant discriminator attribute '%s'\n",
+            virBufferAsprintf(ctxt->debug, "ERROR: missing variant discriminator attribute '%s'\n",
                               variantfield);
-            goto cleanup;
+            return -1;
         }
 
         if (testQEMUSchemaValidateObjectMergeVariant(localroot, variantfield,
-                                                     variantname,
-                                                     schema, debug) < 0)
-            goto cleanup;
+                                                     variantname, ctxt) < 0)
+            return -1;
     }
 
 
@@ -308,33 +294,28 @@ testQEMUSchemaValidateObject(virJSONValuePtr obj,
     if (virJSONValueObjectForeachKeyValue(obj,
                                           testQEMUSchemaValidateObjectMember,
                                           &data) < 0)
-        goto cleanup;
+        return -1;
 
     /* check missing mandatory values */
     if (virJSONValueArrayForeachSteal(data.rootmembers,
                                       testQEMUSchemaValidateObjectMandatoryMember,
                                       &data) < 0) {
-        ret = -2;
-        goto cleanup;
+        return -2;
     }
 
     if (data.missingMandatory)
-        goto cleanup;
+        return -1;
 
-    virBufferAdjustIndent(debug, -3);
-    virBufferAddLit(debug, "} OK");
-    ret = 0;
-
- cleanup:
-    virJSONValueFree(localroot);
-    return ret;
+    virBufferAdjustIndent(ctxt->debug, -3);
+    virBufferAddLit(ctxt->debug, "} OK");
+    return 0;
 }
 
 
 static int
 testQEMUSchemaValidateEnum(virJSONValuePtr obj,
                            virJSONValuePtr root,
-                           virBufferPtr debug)
+                           struct testQEMUSchemaValidateCtxt *ctxt)
 {
     const char *objstr;
     virJSONValuePtr values = NULL;
@@ -342,14 +323,14 @@ testQEMUSchemaValidateEnum(virJSONValuePtr obj,
     size_t i;
 
     if (virJSONValueGetType(obj) != VIR_JSON_TYPE_STRING) {
-        virBufferAddLit(debug, "ERROR: not a string");
+        virBufferAddLit(ctxt->debug, "ERROR: not a string");
         return -1;
     }
 
     objstr = virJSONValueGetString(obj);
 
     if (!(values = virJSONValueObjectGetArray(root, "values"))) {
-        virBufferAsprintf(debug, "ERROR: missing enum values in schema '%s'",
+        virBufferAsprintf(ctxt->debug, "ERROR: missing enum values in schema '%s'",
                           NULLSTR(virJSONValueObjectGetString(root, "name")));
         return -2;
     }
@@ -358,12 +339,12 @@ testQEMUSchemaValidateEnum(virJSONValuePtr obj,
         value = virJSONValueArrayGet(values, i);
 
         if (STREQ_NULLABLE(objstr, virJSONValueGetString(value))) {
-            virBufferAsprintf(debug, "'%s' OK", NULLSTR(objstr));
+            virBufferAsprintf(ctxt->debug, "'%s' OK", NULLSTR(objstr));
             return 0;
         }
     }
 
-    virBufferAsprintf(debug, "ERROR: enum value '%s' is not in schema",
+    virBufferAsprintf(ctxt->debug, "ERROR: enum value '%s' is not in schema",
                       NULLSTR(objstr));
     return -1;
 }
@@ -372,8 +353,7 @@ testQEMUSchemaValidateEnum(virJSONValuePtr obj,
 static int
 testQEMUSchemaValidateArray(virJSONValuePtr objs,
                             virJSONValuePtr root,
-                            virHashTablePtr schema,
-                            virBufferPtr debug)
+                            struct testQEMUSchemaValidateCtxt *ctxt)
 {
     const char *elemtypename = virJSONValueObjectGetString(root, "element-type");
     virJSONValuePtr elementschema;
@@ -381,29 +361,29 @@ testQEMUSchemaValidateArray(virJSONValuePtr objs,
     size_t i;
 
     if (virJSONValueGetType(objs) != VIR_JSON_TYPE_ARRAY) {
-        virBufferAddLit(debug, "ERROR: not an array\n");
+        virBufferAddLit(ctxt->debug, "ERROR: not an array\n");
         return -1;
     }
 
     if (!elemtypename ||
-        !(elementschema = virHashLookup(schema, elemtypename))) {
-        virBufferAsprintf(debug, "ERROR: missing schema for array element type '%s'",
+        !(elementschema = virHashLookup(ctxt->schema, elemtypename))) {
+        virBufferAsprintf(ctxt->debug, "ERROR: missing schema for array element type '%s'",
                          NULLSTR(elemtypename));
         return -2;
     }
 
-    virBufferAddLit(debug, "[\n");
-    virBufferAdjustIndent(debug, 3);
+    virBufferAddLit(ctxt->debug, "[\n");
+    virBufferAdjustIndent(ctxt->debug, 3);
 
     for (i = 0; i < virJSONValueArraySize(objs); i++) {
         obj = virJSONValueArrayGet(objs, i);
 
-        if (testQEMUSchemaValidateRecurse(obj, elementschema, schema, debug) < 0)
+        if (testQEMUSchemaValidateRecurse(obj, elementschema, ctxt) < 0)
             return -1;
-        virBufferAddLit(debug, ",\n");
+        virBufferAddLit(ctxt->debug, ",\n");
     }
-    virBufferAddLit(debug, "] OK");
-    virBufferAdjustIndent(debug, -3);
+    virBufferAddLit(ctxt->debug, "] OK");
+    virBufferAdjustIndent(ctxt->debug, -3);
 
     return 0;
 }
@@ -411,8 +391,7 @@ testQEMUSchemaValidateArray(virJSONValuePtr objs,
 static int
 testQEMUSchemaValidateAlternate(virJSONValuePtr obj,
                                 virJSONValuePtr root,
-                                virHashTablePtr schema,
-                                virBufferPtr debug)
+                                struct testQEMUSchemaValidateCtxt *ctxt)
 {
     virJSONValuePtr members;
     virJSONValuePtr member;
@@ -424,70 +403,114 @@ testQEMUSchemaValidateAlternate(virJSONValuePtr obj,
     int rc;
 
     if (!(members = virJSONValueObjectGetArray(root, "members"))) {
-        virBufferAddLit(debug, "ERROR: missing 'members' for alternate schema");
+        virBufferAddLit(ctxt->debug, "ERROR: missing 'members' for alternate schema");
         return -2;
     }
 
-    virBufferAddLit(debug, "(\n");
-    virBufferAdjustIndent(debug, 3);
-    indent = virBufferGetIndent(debug);
+    virBufferAddLit(ctxt->debug, "(\n");
+    virBufferAdjustIndent(ctxt->debug, 3);
+    indent = virBufferGetIndent(ctxt->debug);
 
     n = virJSONValueArraySize(members);
     for (i = 0; i < n; i++) {
         membertype = NULL;
 
         /* P != NP */
-        virBufferAsprintf(debug, "(alternate %zu/%zu)\n", i + 1, n);
-        virBufferAdjustIndent(debug, 3);
+        virBufferAsprintf(ctxt->debug, "(alternate %zu/%zu)\n", i + 1, n);
+        virBufferAdjustIndent(ctxt->debug, 3);
 
         if (!(member = virJSONValueArrayGet(members, i)) ||
             !(membertype = virJSONValueObjectGetString(member, "type")) ||
-            !(memberschema = virHashLookup(schema, membertype))) {
-            virBufferAsprintf(debug, "ERROR: missing schema for alternate type '%s'",
+            !(memberschema = virHashLookup(ctxt->schema, membertype))) {
+            virBufferAsprintf(ctxt->debug, "ERROR: missing schema for alternate type '%s'",
                               NULLSTR(membertype));
             return -2;
         }
 
-        rc = testQEMUSchemaValidateRecurse(obj, memberschema, schema, debug);
+        rc = testQEMUSchemaValidateRecurse(obj, memberschema, ctxt);
 
-        virBufferAddLit(debug, "\n");
-        virBufferSetIndent(debug, indent);
-        virBufferAsprintf(debug, "(/alternate %zu/%zu)\n", i + 1, n);
+        virBufferAddLit(ctxt->debug, "\n");
+        virBufferSetIndent(ctxt->debug, indent);
+        virBufferAsprintf(ctxt->debug, "(/alternate %zu/%zu)\n", i + 1, n);
 
         if (rc == 0) {
-            virBufferAdjustIndent(debug, -3);
-            virBufferAddLit(debug, ") OK");
+            virBufferAdjustIndent(ctxt->debug, -3);
+            virBufferAddLit(ctxt->debug, ") OK");
             return 0;
         }
     }
 
-    virBufferAddLit(debug, "ERROR: no alternate type was matched");
+    virBufferAddLit(ctxt->debug, "ERROR: no alternate type was matched");
     return -1;
 }
 
 
+static int
+testQEMUSchemaValidateDeprecated(virJSONValuePtr root,
+                                 const char *name,
+                                 struct testQEMUSchemaValidateCtxt *ctxt)
+{
+    virJSONValuePtr features = virJSONValueObjectGetArray(root, "features");
+    size_t nfeatures;
+    size_t i;
+
+    if (!features)
+        return 0;
+
+    nfeatures = virJSONValueArraySize(features);
+
+    for (i = 0; i < nfeatures; i++) {
+        virJSONValuePtr cur = virJSONValueArrayGet(features, i);
+        const char *curstr;
+
+        if (!cur ||
+            !(curstr = virJSONValueGetString(cur))) {
+            virBufferAsprintf(ctxt->debug, "ERROR: features of '%s' are malformed", name);
+            return -2;
+        }
+
+        if (STREQ(curstr, "deprecated")) {
+            if (ctxt->allowDeprecated) {
+                virBufferAsprintf(ctxt->debug, "WARNING: '%s' is deprecated", name);
+                if (virTestGetVerbose())
+                    g_fprintf(stderr, "\nWARNING: '%s' is deprecated\n", name);
+                return 0;
+            } else {
+                virBufferAsprintf(ctxt->debug, "ERROR: '%s' is deprecated", name);
+                return -1;
+            }
+        }
+    }
+
+    return 0;
+}
+
+
 static int
 testQEMUSchemaValidateRecurse(virJSONValuePtr obj,
                               virJSONValuePtr root,
-                              virHashTablePtr schema,
-                              virBufferPtr debug)
+                              struct testQEMUSchemaValidateCtxt *ctxt)
 {
     const char *n = virJSONValueObjectGetString(root, "name");
     const char *t = virJSONValueObjectGetString(root, "meta-type");
+    int rc;
+
+    if ((rc = testQEMUSchemaValidateDeprecated(root, n, ctxt)) < 0)
+        return rc;
 
     if (STREQ_NULLABLE(t, "builtin")) {
-        return testQEMUSchemaValidateBuiltin(obj, root, debug);
+        return testQEMUSchemaValidateBuiltin(obj, root, ctxt);
     } else if (STREQ_NULLABLE(t, "object")) {
-        return testQEMUSchemaValidateObject(obj, root, schema, debug);
+        return testQEMUSchemaValidateObject(obj, root, ctxt);
     } else if (STREQ_NULLABLE(t, "enum")) {
-        return testQEMUSchemaValidateEnum(obj, root, debug);
+        return testQEMUSchemaValidateEnum(obj, root, ctxt);
     } else if (STREQ_NULLABLE(t, "array")) {
-        return testQEMUSchemaValidateArray(obj, root, schema, debug);
+        return testQEMUSchemaValidateArray(obj, root, ctxt);
     } else if (STREQ_NULLABLE(t, "alternate")) {
-        return testQEMUSchemaValidateAlternate(obj, root, schema, debug);
+        return testQEMUSchemaValidateAlternate(obj, root, ctxt);
     }
 
-    virBufferAsprintf(debug,
+    virBufferAsprintf(ctxt->debug,
                       "qapi schema meta-type '%s' of type '%s' not handled\n",
                       NULLSTR(t), NULLSTR(n));
     return -2;
@@ -511,9 +534,115 @@ int
 testQEMUSchemaValidate(virJSONValuePtr obj,
                        virJSONValuePtr root,
                        virHashTablePtr schema,
+                       bool allowDeprecated,
                        virBufferPtr debug)
 {
-    return testQEMUSchemaValidateRecurse(obj, root, schema, debug);
+    struct testQEMUSchemaValidateCtxt ctxt = { .schema = schema,
+                                               .debug = debug,
+                                               .allowDeprecated = allowDeprecated };
+
+    return testQEMUSchemaValidateRecurse(obj, root, &ctxt);
+}
+
+
+/**
+ * testQEMUSchemaValidateCommand:
+ * @command: command to validate
+ * @arguments: arguments of @command to validate
+ * @schema: hash table containing schema entries
+ * @allowDeprecated: don't fails schema validation if @command or one of @arguments
+ *                   is deprecated
+ * @allowRemoved: skip validation fully if @command was not found
+ * @debug: a virBuffer which will be filled with debug information if provided
+ *
+ * Validates whether @command and its @arguments conform to the QAPI schema
+ * passed in via @schema. Returns 0, if the command and args match @schema,
+ * -1 if it does not and -2 if there is a problem with the schema or with
+ *  internals.
+ *
+ * @allowRemoved should generally be used only if it's certain that there's a
+ * replacement of @command in place.
+ *
+ * @debug is filled with information regarding the validation process
+ */
+int
+testQEMUSchemaValidateCommand(const char *command,
+                              virJSONValuePtr arguments,
+                              virHashTablePtr schema,
+                              bool allowDeprecated,
+                              bool allowRemoved,
+                              virBufferPtr debug)
+{
+    struct testQEMUSchemaValidateCtxt ctxt = { .schema = schema,
+                                               .debug = debug,
+                                               .allowDeprecated = allowDeprecated };
+    g_autofree char *schemapatharguments = g_strdup_printf("%s/arg-type", command);
+    g_autoptr(virJSONValue) emptyargs = NULL;
+    virJSONValuePtr schemarootcommand;
+    virJSONValuePtr schemarootarguments;
+    int rc;
+
+    if (virQEMUQAPISchemaPathGet(command, schema, &schemarootcommand) < 0 ||
+        !schemarootcommand) {
+        if (allowRemoved)
+            return 0;
+
+        virBufferAsprintf(debug, "ERROR: command '%s' not found in the schema", command);
+        return -1;
+    }
+
+    if ((rc = testQEMUSchemaValidateDeprecated(schemarootcommand, command, &ctxt)) < 0)
+        return rc;
+
+    if (!arguments)
+        arguments = emptyargs = virJSONValueNewObject();
+
+    if (virQEMUQAPISchemaPathGet(schemapatharguments, schema, &schemarootarguments) < 0 ||
+        !schemarootarguments) {
+        virBufferAsprintf(debug, "ERROR: failed to look up 'arg-type' of  '%s'", command);
+        return -1;
+    }
+
+    return testQEMUSchemaValidateRecurse(arguments, schemarootarguments, &ctxt);
+}
+
+
+static virJSONValuePtr
+testQEMUSchemaLoadReplies(const char *filename)
+{
+    g_autofree char *caps = NULL;
+    char *schemaReply;
+    char *end;
+    g_autoptr(virJSONValue) reply = NULL;
+    virJSONValuePtr schema = NULL;
+
+    if (virTestLoadFile(filename, &caps) < 0)
+        return NULL;
+
+    if (!(schemaReply = strstr(caps, "\"execute\": \"query-qmp-schema\"")) ||
+        !(schemaReply = strstr(schemaReply, "\n\n")) ||
+        !(end = strstr(schemaReply + 2, "\n\n"))) {
+        VIR_TEST_VERBOSE("failed to find reply to 'query-qmp-schema' in '%s'",
+                         filename);
+        return NULL;
+    }
+
+    schemaReply += 2;
+    *end = '\0';
+
+    if (!(reply = virJSONValueFromString(schemaReply))) {
+        VIR_TEST_VERBOSE("failed to parse 'query-qmp-schema' reply from '%s'",
+                         filename);
+        return NULL;
+    }
+
+    if (!(schema = virJSONValueObjectStealArray(reply, "return"))) {
+        VIR_TEST_VERBOSE("missing qapi schema data in reply in '%s'",
+                         filename);
+        return NULL;
+    }
+
+    return schema;
 }
 
 
@@ -524,14 +653,9 @@ testQEMUSchemaValidate(virJSONValuePtr obj,
  * replies file used for qemucapabilitiestest for the x86_64 architecture.
  */
 virJSONValuePtr
-testQEMUSchemaGetLatest(const char* arch)
+testQEMUSchemaGetLatest(const char *arch)
 {
-    char *capsLatestFile = NULL;
-    char *capsLatest = NULL;
-    char *schemaReply;
-    char *end;
-    virJSONValuePtr reply = NULL;
-    virJSONValuePtr schema = NULL;
+    g_autofree char *capsLatestFile = NULL;
 
     if (!(capsLatestFile = testQemuGetLatestCapsForArch(arch, "replies"))) {
         VIR_TEST_VERBOSE("failed to find latest caps replies");
@@ -540,42 +664,12 @@ testQEMUSchemaGetLatest(const char* arch)
 
     VIR_TEST_DEBUG("replies file: '%s'", capsLatestFile);
 
-    if (virTestLoadFile(capsLatestFile, &capsLatest) < 0)
-        goto cleanup;
-
-    if (!(schemaReply = strstr(capsLatest, "\"execute\": \"query-qmp-schema\"")) ||
-        !(schemaReply = strstr(schemaReply, "\n\n")) ||
-        !(end = strstr(schemaReply + 2, "\n\n"))) {
-        VIR_TEST_VERBOSE("failed to find reply to 'query-qmp-schema' in '%s'",
-                         capsLatestFile);
-        goto cleanup;
-    }
-
-    schemaReply += 2;
-    *end = '\0';
-
-    if (!(reply = virJSONValueFromString(schemaReply))) {
-        VIR_TEST_VERBOSE("failed to parse 'query-qmp-schema' reply from '%s'",
-                         capsLatestFile);
-        goto cleanup;
-    }
-
-    if (!(schema = virJSONValueObjectStealArray(reply, "return"))) {
-        VIR_TEST_VERBOSE("missing qapi schema data in reply in '%s'",
-                         capsLatestFile);
-        goto cleanup;
-    }
-
- cleanup:
-    VIR_FREE(capsLatestFile);
-    VIR_FREE(capsLatest);
-    virJSONValueFree(reply);
-    return schema;
+    return testQEMUSchemaLoadReplies(capsLatestFile);
 }
 
 
 virHashTablePtr
-testQEMUSchemaLoad(const char* arch)
+testQEMUSchemaLoadLatest(const char *arch)
 {
     virJSONValuePtr schema;
 
@@ -584,3 +678,15 @@ testQEMUSchemaLoad(const char* arch)
 
     return virQEMUQAPISchemaConvert(schema);
 }
+
+
+virHashTablePtr
+testQEMUSchemaLoad(const char *filename)
+{
+    virJSONValuePtr schema;
+
+    if (!(schema = testQEMUSchemaLoadReplies(filename)))
+        return NULL;
+
+    return virQEMUQAPISchemaConvert(schema);
+}
