@@ -193,8 +193,7 @@ test4(const void *data G_GNUC_UNUSED)
 
     /* 0. empty set */
 
-    if (!(bitmap = virBitmapNewEmpty()))
-        goto error;
+    bitmap = virBitmapNewEmpty();
 
     if (virBitmapNextSetBit(bitmap, -1) != -1)
         goto error;
@@ -632,12 +631,9 @@ test11(const void *opaque)
 static int
 test12(const void *opaque G_GNUC_UNUSED)
 {
-    virBitmapPtr map = NULL;
+    virBitmapPtr map = virBitmapNewEmpty();
     int ret = -1;
 
-    if (!(map = virBitmapNewEmpty()))
-        return -1;
-
     TEST_MAP(0, "");
 
     if (virBitmapSetBitExpand(map, 128) < 0)
